Include "geophysique_param.geo";

Group {
  Layer1 = Region[OMEGA_1];
  Layer2 = Region[OMEGA_2];
  Layer3 = Region[OMEGA_3];
  Layer4 = Region[OMEGA_4];
  Bnd1 = Region[GAMMAINF_1];
  Bnd2 = Region[GAMMAINF_2];
  Bnd3 = Region[GAMMAINF_3];
  Bnd4 = Region[GAMMAINF_4];

  GammaInf = Region[{Bnd1,Bnd2,Bnd3,Bnd4}];
  Omega = Region[{Layer1,Layer2,Layer3,Layer4}];
  GammaD = Region[GAMMAD];
  Sigma = Region[{}];
}

Function {
  DefineConstant[
    c1 = {1500, Name "Input/c1"} ,
    c2 = {1500, Name "Input/c2"} ,
    c3 = {1500, Name "Input/c3"} ,
    c4 = {1500, Name "Input/c4"}
  ];
  c[Region[{Layer1,Bnd1}]] = c1;
  c[Region[{Layer2,Bnd2}]] = c2;
  c[Region[{Layer3,Bnd3}]] = c3;
  c[Region[{Layer4,Bnd4}]] = c4;
  t_min = 0;
  t_max = 1;
  dt = 1/100;
}

Jacobian {
  { Name JVol ; Case { { Region All ; Jacobian Vol ; } } }
  { Name JSur ; Case { { Region All ; Jacobian Sur ; } } }
  { Name JLin ; Case { { Region All ; Jacobian Lin ; } } }
}

Integration {
  { Name I1 ;
    Case { { Type Gauss ; Case {
          { GeoElement Point ; NumberOfPoints  1 ; }
          { GeoElement Line ; NumberOfPoints  2 ; }
          { GeoElement Triangle ; NumberOfPoints  6 ; }
          { GeoElement Tetrahedron ; NumberOfPoints  4 ; }
          { GeoElement Quadrangle ; NumberOfPoints  4 ; }
          { GeoElement Hexahedron ; NumberOfPoints  14 ; }
        } } }
  }
}

Constraint{
  { Name Dirichlet_uinc ;
    Case {
      { Region GammaD ; Value 1;
        TimeFunction Cos[2*Pi*Freq*$Time];
      }
    }
  }
}

FunctionSpace {
  { Name Hgrad_u; Type Form0 ;
    BasisFunction {
      { Name sn ; NameOfCoef un ; Function BF_Node ;
        Support Region[ {Omega, GammaInf, Sigma, GammaD} ] ; Entity NodesOf[ All ] ; } }
    Constraint {
      { NameOfCoef un ; EntityType NodesOf ; NameOfConstraint Dirichlet_uinc ; }
    }
  }
}

Formulation {
  { Name Full ; Type FemEquation ;
    Quantity {
      { Name u ; Type Local ; NameOfSpace Hgrad_u; }
    }
    Equation {
      Galerkin { [ - Dof{Grad u} , {Grad u} ] ;
	In Omega; Jacobian JVol ; Integration I1 ; }
      Galerkin { DtDtDof [ - 1/c[]^2 * Dof{u} , {u} ] ;
	In Omega; Jacobian JVol ; Integration I1 ; }
      Galerkin { DtDof [ - 1/c[] * Dof{u} , {u} ] ;
	In GammaInf; Jacobian JSur ; Integration I1 ; }
    }
  }
}

Resolution {
  { Name Full_f;
    System {
      { Name Helmholtz ; NameOfFormulation Full ; Type Complex; Frequency Freq; }
    }
    Operation {
      Generate[Helmholtz] ;
      Solve[Helmholtz] ;
      SaveSolution[Helmholtz];
    }
  }
  { Name Full_t;
    System {
      { Name Helmholtz ; NameOfFormulation Full ; }
    }
    Operation {
      InitSolution[Helmholtz];
      InitSolution[Helmholtz];
      // fast:
      GenerateSeparate[Helmholtz];
      TimeLoopNewmark[t_min,t_max,dt,0.25,0.5]{
        Update[Helmholtz] ;
        SolveAgain[Helmholtz] ;
        SaveSolution[Helmholtz];
      }
      // slow:
      /*
      TimeLoopNewmark[t_min,t_max,dt,0.25,0.5]{
        Generate[Helmholtz] ;
        SolveAgain[Helmholtz] ;
        SaveSolution[Helmholtz];
      }
      */
    }
  }
}

PostProcessing {
  { Name Full ; NameOfFormulation Full ;
    PostQuantity {
      { Name u ; Value { Term { [ {u} ] ; In Region[ {Omega} ]; Jacobian JVol ; } } }
    }
  }
}

PostOperation {
  { Name Full ; NameOfPostProcessing Full;
    Operation {
      Print[ u, OnElementsOf Omega, File "u.pos"] ;
    }
  }
}

DefineConstant[
  //R_ = {"Full", Name "GetDP/1ResolutionChoices", Visible 0},
  C_ = {"-solve -pos -v2 -bin", Name "GetDP/9ComputeCommand", Visible 1}
  //P_ = {"", Name "GetDP/2PostOperationChoices", Visible 0}
];

